local MODNAME = ...

-------------------------------------------------------------------------------
-- Localized Lua globals
-------------------------------------------------------------------------------

local _G = getfenv(0)
local floor		= _G.floor
local format	= _G.format
local ipairs	= _G.ipairs
local pairs		= _G.pairs
local select	= _G.select
local tinsert	= _G.tinsert
local tonumber	= _G.tonumber
local tostring	= _G.tostring
local sort		= _G.sort
local type		= _G.type
local wipe		= _G.wipe

local Syllabus = LibStub("AceAddon-3.0"):GetAddon(MODNAME)

local BL = LibStub("LibBabble-Faction-3.0"):GetLookupTable()
local BR = LibStub("LibBabble-Faction-3.0"):GetReverseLookupTable()

-- Contants defined in Sullabus.lua
local ABILITY_NAME_TO_SPELLID					= Syllabus.ABILITY_NAME_TO_SPELLID
local MY_FACTION_FLAG							= Syllabus.MY_FACTION_FLAG
local ZONE_NAMES									= Syllabus.ZONE_NAMES

-- Functions defined in Sullabus.lua
local function err(...) Syllabus:err(...) end
local IsTableEmpty								= Syllabus.IsTableEmpty
local GetLocalProfName							= Syllabus.GetLocalProfName
local GetStaticProfName							= Syllabus.GetStaticProfName

-------------------------------------------------------------------------------
-- Utility Function
-------------------------------------------------------------------------------

function Syllabus:Save()
	self:SaveSpells()
	self:SaveTrainers()
end

function Syllabus:ClearAllSaves()
	Syllabus.db.global.save_spell = nil
	Syllabus.db.global.save_trainer_without_ref = nil
	Syllabus.db.global.save_trainer_without_id = nil
	Syllabus.db.global.save_trainer = nil
end

function Syllabus:FixSpellRep()
	-- Use the trainers' rep to set the spells' rep
	local trainers_db = Syllabus.db.global.trainers
	local skills_db = Syllabus.db.global.skills

	for type_name, skill_type in pairs(skills_db) do
		if type_name == "all_classes" then
			for spell_id, spell in pairs(skill_type) do
				for trainer_id in pairs(spell.trainers) do
					if trainers_db[trainer_id].rep then
						spell.valid_rep[trainers_db[trainer_id].rep] = true
					end
				end
			end
		else
			for skill_name, skill in pairs(skill_type) do
				for spell_id, spell in pairs(skill) do
					for trainer_id in pairs(spell.trainers) do
						if trainers_db[trainer_id].rep then
							spell.valid_rep[trainers_db[trainer_id].rep] = true
						end
					end
				end
			end
		end
	end

end

function Syllabus:FixTrainerRef()
	local trainers_db = Syllabus.db.global.trainers
	local skills_db = Syllabus.db.global.skills
	local name_to_id = {}

	for trainer_id, trainer in pairs(trainers_db) do
		if not IsTableEmpty(trainer) then
			local name = type(trainer_id) == "string" and trainer_id or trainer.name
			local id = type(trainer_id) == "number" and trainer_id or trainer.id
			if name and id then name_to_id[name] = id end
		end
	end

	for type_name, skill_type in pairs(skills_db) do
		if type_name == "all_classes" then
			for spell_id, spell in pairs(skill_type) do
				for trainer_id in pairs(spell.trainers) do
					if type(trainer_id) == "string" then
						if name_to_id[trainer_id] then
							spell.trainers[name_to_id[trainer_id]] = true
							spell.trainers[trainer_id] = nil
						end
					end
				end
			end
		else
			for skill_name, skill in pairs(skill_type) do
				for spell_id, spell in pairs(skill) do
					for trainer_id in pairs(spell.trainers) do
						if type(trainer_id) == "string" then
							if name_to_id[trainer_id] then
								spell.trainers[name_to_id[trainer_id]] = true
								spell.trainers[trainer_id] = nil
							end
						end
					end
				end
			end
		end
	end

end
function Syllabus:FindNearTrainerToVisit()
	self:FindTrainersWithoutRef(true, true, true)
end

function Syllabus:FindTrainersWithoutRef(faction, print_list, current_zone)
	local trainers_db = Syllabus.db.global.trainers
	local skills_db = Syllabus.db.global.skills
	local trainer_ref = {}

	_G.SetMapToCurrentZone() -- Make sure were are looking at the right zone
	local continent_id, zone_id = _G.GetCurrentMapContinent(), _G.GetCurrentMapZone()

	-- Collect all the reference to trainers from the spells
	for type_name, skill_type in pairs(skills_db) do
		if type_name == "all_classes" then
			for spell_id, spell in pairs(skill_type) do
				for trainer_id in pairs(spell.trainers) do
					trainer_ref[trainer_id] = "Riding"
				end
			end
		else
			for skill_name, skill in pairs(skill_type) do
				for spell_id, spell in pairs(skill) do
					for trainer_id in pairs(spell.trainers) do
						trainer_ref[trainer_id] = skill_name
					end
				end
			end
		end
	end

	-- All the lonely people, where do they all come from?
	Syllabus.db.global.save_trainer_without_ref = Syllabus.db.global.save_trainer_without_ref or {}
	local save = Syllabus.db.global.save_trainer_without_ref
	wipe(save)

	for trainer_id, trainer in pairs(trainers_db) do
		if not trainer_ref[trainer_id] and
			not IsTableEmpty(trainer) and
			(not faction or trainer[MY_FACTION_FLAG]) and
			(not current_zone or (continent_id == trainer.continent_id and zone_id == trainer.zone_id))
		then
			tinsert(save, trainer_id)
		end
	end

	sort(save)
	if print_list then
		for i=1,#save do
			self:Print(("%s - %s (%.1f, %.1f)"):format(
				tostring(save[i]),
				tostring(trainers_db[save[i]].name),
				tostring(trainers_db[save[i]].x * 100 or 0),
				tostring(trainers_db[save[i]].y * 100 or 0)
			))
		end
	end

	self:Print(("%d trainers without ref were found"):format(#save))
end

function Syllabus:FindTrainersWithoutID(faction, print_list)
	local trainers_db = Syllabus.db.global.trainers

	Syllabus.db.global.save_trainer_without_id = Syllabus.db.global.save_trainer_without_id or {}
	local save = Syllabus.db.global.save_trainer_without_id
	wipe(save)

	for trainer_id, trainer in pairs(trainers_db) do
		if type(trainer_id) == "string" and not IsTableEmpty(trainer) and (not faction or trainer[MY_FACTION_FLAG]) then
			tinsert(save, trainer_id)
		end
	end

	sort(save)
	if print_list then
		for i=1,#save do
			self:Print(save[i])
		end
	end

	self:Print(("%d trainers without id were found"):format(#save))
end

function Syllabus:CleanCummonClassSpells()
	local skills_db = Syllabus.db.global.skills

	local is_commun_spell = {}
	local all_classes = skills_db["all_classes"]
	local classes = skills_db["classes"]

	for spell_id in pairs(all_classes) do
		is_commun_spell[spell_id] = true
	end

	for class, class_spells in pairs(classes) do
		for spell_id, spell in pairs(class_spells) do
			if is_commun_spell[spell_id] and not IsTableEmpty(spell) then
				wipe(spell)
			end
		end
	end
end


-------------------------------------------------------------------------------
-- Learnable spells
-------------------------------------------------------------------------------

function Syllabus:SaveSpells()
	local skills_db = Syllabus.db.global.skills

	Syllabus.db.global.save_spell = Syllabus.db.global.save_spell or {}
	wipe(Syllabus.db.global.save_spell)

	Syllabus.db.global.save_spell_ref = Syllabus.db.global.save_spell_ref or {}
	wipe(Syllabus.db.global.save_spell_ref)

	local sorted_trainers = {}
	local sort_trainer_id = function(a,b)
		local na = tonumber(a) or 0
		local nb = tonumber(b) or 0

		if na ~= nb then
			return na < nb
		else
			return a < b
		end
	end

	for type_name, skill_type in pairs(skills_db) do
		if type_name == "all_classes" then
			for spell_id, spell in pairs(skill_type) do
				if not IsTableEmpty(spell) then
					local faction = (spell.is_alliance and 1 or 0) + (spell.is_horde and 2 or 0)
					local add_spell = ("AddSpell( [[%s]], nil, %s, %s, %s, %s, %s, %s, [[%s]], %s ) -- %s"):format(
						type_name,
						spell_id,
						tostring(spell.level_req),
						tostring(spell.rank_req),
						tostring(spell.num_reqs),
						tostring(spell.requires and ABILITY_NAME_TO_SPELLID[spell.requires] or spell.requires),
						tostring(spell.cost),
						tostring(spell.icon ~= _G.GetSpellTexture(spell_id) and spell.icon or nil),
						faction,
						tostring(spell.name)
					)
					tinsert(Syllabus.db.global.save_spell, add_spell)

					-- Sort the trainers before building the string
					wipe(sorted_trainers)
					for trainer_id in pairs(spell.trainers) do
						tinsert(sorted_trainers, trainer_id)
					end
					sort(sorted_trainers, sort_trainer_id)

					local add_trainer_ref = ("AddTrainerRef( [[%s]], nil, %s"):format(type_name, spell_id)
					for _, trainer_id in ipairs(sorted_trainers) do
						add_trainer_ref = add_trainer_ref .. ", " .. trainer_id
					end
					add_trainer_ref = add_trainer_ref .. ")"

					tinsert(Syllabus.db.global.save_spell_ref, add_trainer_ref)
				end
			end
		else
			for skill_name, skill in pairs(skill_type) do
				for spell_id, spell in pairs(skill) do
					if not IsTableEmpty(spell) then
						local add_spell = ("AddSpell( [[%s]], [[%s]], %s, %s, %s, %s, %s, %s, [[%s]] ) -- %s"):format(
							type_name,
							GetStaticProfName(skill_name, true),
							spell_id,
							tostring(spell.level_req),
							tostring(spell.rank_req),
							tostring(spell.num_reqs),
							tostring(spell.requires and ABILITY_NAME_TO_SPELLID[spell.requires] or spell.requires),
							tostring(spell.cost),
							tostring(spell.icon ~= _G.GetSpellTexture(spell_id) and spell.icon or nil),
							tostring(spell.name)
						)
						tinsert(Syllabus.db.global.save_spell, add_spell)

						-- Sort the trainers before building the string
						wipe(sorted_trainers)
						for trainer_id in pairs(spell.trainers) do
							tinsert(sorted_trainers, trainer_id)
						end
						sort(sorted_trainers, sort_trainer_id)

						local add_trainer_ref = ("AddTrainerRef( [[%s]], [[%s]], %s"):format(type_name, GetStaticProfName(skill_name, true), spell_id)
						for _, trainer_id in ipairs(sorted_trainers) do
							add_trainer_ref = add_trainer_ref .. ", " .. trainer_id
						end
						add_trainer_ref = add_trainer_ref .. ")"

						tinsert(Syllabus.db.global.save_spell_ref, add_trainer_ref)
					end
				end
			end
		end
	end

	sort(Syllabus.db.global.save_spell)
	sort(Syllabus.db.global.save_spell_ref)
end

-- AddSpell( "all_classes", nil, 90267, 60, 225, 0, nil, 2500000, "nil" ) -- Flight Master's License
-- AddSpell( "professions", "ENGINEERING", 12715, nil, 205, 1, 20222, 1000, "Interface\\Icons\\INV_Scroll_03" ) -- Goblin Rocket Fuel Recipe
function Syllabus:AddSpell(type_name, skill_name, spell_id, level_req, rank_req, num_reqs, requires, cost, icon)
	local skills_db = Syllabus.db.global.skills

	local local_skill_name = type_name == "professions" and GetLocalProfName(skill_name) or skill_name
	local spell
	if type_name == "all_classes"then
		spell = skills_db[type_name][spell_id]
	else
		spell = skills_db[type_name][local_skill_name][spell_id]
	end

	if spell.level_req and spell.level_req ~= level_req then
		err("Level requirement missmatch: %s (existing) ~= %s (new)", spell.level_req, level_req)
	else
		spell.level_req = level_req
	end

	if spell.rank_req and spell.rank_req ~= rank_req then
		err("Rank requirement missmatch: %s (existing) ~= %s (new)", spell.rank_req, rank_req)
	else
		spell.rank_req = rank_req
	end

	if spell.num_reqs and spell.num_reqs ~= num_reqs then
		err("Number of requirement missmatch: %s (existing) ~= %s (new)", spell.num_reqs, num_reqs)
	else
		spell.num_reqs = num_reqs
	end

	if spell.requires and spell.requires ~= requires then
		err("Required spell missmatch: %s (existing) ~= %s (new)", spell.requires, requires)
	else
		spell.requires = requires
	end

	if spell.cost and spell.cost ~= cost then
		err("Training cost missmatch: %s (existing) ~= %s (new)", spell.cost, cost)
	else
		spell.cost = cost
	end

	if spell.icon and spell.icon ~= icon then
		err("Icon texture missmatch: %s (existing) ~= %s (new)", spell.icon, icon)
	else
		spell.icon = icon
	end

end

function Syllabus:AddSpells()
end

-------------------------------------------------------------------------------
-- Trainners
-------------------------------------------------------------------------------

function Syllabus:SaveTrainers()
	local trainers_db = Syllabus.db.global.trainers

	Syllabus.db.global.save_trainer = Syllabus.db.global.save_trainer or {}
	wipe(Syllabus.db.global.save_trainer)

	for trainer_id, trainer in pairs(trainers_db) do
		if not IsTableEmpty(trainer) then
			local id = type(trainer_id) == "number" and trainer_id or trainer.id
			local name = type(trainer_id) == "string" and trainer_id or trainer.name

			local x = trainer.x and trainer.x * 100 or 0
			local y = trainer.y and trainer.y * 100 or 0
			local faction = (trainer.is_alliance and 1 or 0) + (trainer.is_horde and 2 or 0)

			local zone_name = ZONE_NAMES[trainer.continent_id] and ZONE_NAMES[trainer.continent_id][trainer.zone_id]

			local add_string = ('AddTrainer( %s, [[%s]], %s, %s, %.1f, %.1f, [[%s]], %d ) -- zone: %s'):format(
				tostring(id),
				tostring(name),
				tostring(trainer.continent_id),
				tostring(trainer.zone_id),
				x,
				y,
				tostring(BR[trainer.rep]),
				faction,
				tostring(zone_name)
			)

			tinsert(Syllabus.db.global.save_trainer, add_string)
		end
	end

	sort(Syllabus.db.global.save_trainer)

end

local function AddTrainer( id, name, continent_id, zone_id, x, y, rep, faction )
	local trainers_db = Syllabus.db.global.trainers

	-- name and rep needs to be localized, rep can be done using LibBabble-Faction-3.0
	if not id then
		err("Missing trainer id for %s",name)
		return
	end

	if not (trainers_db[name] or trainers_db[id]) then
		err("New trainer: %s - %s", id, name)
	end

	if not IsTableEmpty(trainers_db[name]) then
		-- Check to see if there is a name entry
		trainers_db[id] = trainers_db[name]
		trainers_db[name] = nil
	end

	local trainer =  trainers_db[id]

	if trainer.id then
		if trainer.id ~= id then
			err("Trainer ID missmatch: existing ID = %s, new ID = %s", trainer.id, id)
			return
		end
	else
		trainer.id = id
	end

	if trainer.name then
		if trainer.name ~= name then
			err("Trainer name missmatch: existing name = %s, new name = %s", trainer.name, name)
			return
		end
	else
		trainer.name = name
	end

	if trainer.continent_id then
		if trainer.continent_id ~= continent_id then
			err("Trainer continent_id missmatch: existing id = %s, new is = %s", trainer.continent_id, continent_id)
			return
		end
	else
		trainer.continent_id = continent_id
	end

	if trainer.zone_id then
		if trainer.zone_id ~= zone_id then
			err("Trainer zone_id missmatch: existing id = %s, new is = %s", trainer.zone_id, zone_id)
			return
		end
	else
		trainer.zone_id = zone_id
	end

	if x == 0 then
		err("Invalid coordinates for %s - %s: %s, %s", id, name, x, y)
	else
		if not trainer.x or trainer.x == 0 then
			trainer.x = x / 100
			trainer.y = y / 100
		else
			local my_x = floor(trainer.x*1000 + .5)/10
			local my_y = floor(trainer.y*1000 + .5)/10

			if my_x ~= x then
				--err("Different x values for %s: (%s,%s) ~= (%s,%s)", tostring(name), tostring(my_x), tostring(my_y), tostring(x), tostring(y))
			end

			if my_y ~= y then
				--err("Different y values for %s: (%s,%s) ~= (%s,%s)", tostring(name), tostring(my_x), tostring(my_y), tostring(x), tostring(y))
			end
		end
	end

	if trainer.rep then
		if trainer.rep ~= BL[rep] then
			err("Trainer reputation missmatch: existing rep = %s, new rep = %s", trainer.rep, BL[rep])
			return
		end
	else
		if rep ~= 'nil' then
			trainer.rep = BL[rep]
		else
			err("Missing rep for %s (%s)",id,name)
		end
	end

	if faction == 3 then
		trainer.is_alliance = true
		trainer.is_horde = true
	elseif faction == 1 then
		trainer.is_alliance = true
	elseif faction == 2 then
		trainer.is_horde = true
	else
		err("Unknown faction for %s: %s", name, faction)
	end

end

function Syllabus:AddTrainers()

	AddTrainer( 17246, "\"Cookie\" McWeaksauce", 1, 4, 46.7, 70.5, "Stormwind", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 20914, "Aalun", 1, 23, 81.3, 52.8, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  3967, "Aayndia Floralwind", 1, 2, 35.9, 52.1, "Darnassus", 1 ) -- zone: Ashenvale
	AddTrainer( 47382, "Adam Hossack", 2, 35, 75.8, 37.6, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  1632, "Adele Fielder", 2, 10, 46.4, 62.1, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 26977, "Adelene Sunlance", 4, 1, 41.2, 53.9, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer( 33630, "Aelthin", 3, 6, 38.5, 71.1, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 16771, "Ahonan", 1, 23, 55.2, 82.9, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  4160, "Ainethil", 1, 7, 55.0, 23.8, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 16728, "Akham", 1, 23, 67.3, 74.5, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  3606, "Alanna Raveneye", 1, 22, 36.8, 34.2, "Darnassus", 1 ) -- zone: Teldrassil
	AddTrainer( 28694, "Alard Schmied", 4, 3, 45.3, 28.0, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 50010, "Alaressa", 3, 6, 51.9, 51.8, "The Scryers", 1 ) -- zone: Shattrath City
	AddTrainer( 18802, "Alchemist Gribble", 3, 2, 53.8, 65.8, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer( 33674, "Alchemist Kanhu", 3, 6, 38.8, 29.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer(  1215, "Alchemist Mallory", 2, 10, 39.8, 48.3, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer(  4900, "Alchemist Narett", 1, 10, 64.0, 47.7, nil, 1 ) -- zone: Dustwallow Marsh
	AddTrainer( 33608, "Alchemy", 3, 6, 44.3, 90.5, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer(  4210, "Alegorn", 1, 7, 50.0, 36.5, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 15501, "Aleinia", 2, 11, 48.5, 47.5, "Silvermoon City", 2 ) -- zone: Eversong Woods
	AddTrainer( 26997, "Alestos", 4, 1, 57.5, 72.3, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 16662, "Alestus", 2, 24, 77.6, 71.3, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  5173, "Alexander Calder", 2, 14, 50.1, 7.3, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 26964, "Alexandra McQueen", 4, 6, 79.4, 30.7, "The Hand of Vengeance", 2 ) -- zone: Howling Fjord
	AddTrainer( 26990, "Alexis Marlowe", 4, 1, 57.6, 71.6, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 34786, "Alice Rigsdale", 2, 11, 52.8, 56.1, nil, 2 ) -- zone: Eversong Woods
	AddTrainer( 34785, "Alnar Whitebough", 1, 23, 73.2, 56.4, nil, 1 ) -- zone: The Exodar
	AddTrainer( 29194, "Amal'thazad", 2, 9, 80.4, 47.9, "Knights of the Ebon Blade", 3 ) -- zone: Eastern Plaguelands
	AddTrainer(  4568, "Anastasia Hartwell", 2, 35, 84.8, 10.4, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 22477, "Anchorite Ensham", 3, 7, 30.8, 75.9, "The Sha'tar", 3 ) -- zone: Terokkar Forest
	AddTrainer( 17214, "Anchorite Fateema", 1, 4, 48.5, 51.8, "Exodar", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 17424, "Anchorite Paetheus", 1, 5, 54.7, 54.0, "Exodar", 1 ) -- zone: Bloodmyst Isle
	AddTrainer( 23734, "Anchorite Yazmina", 4, 6, 59.5, 62.3, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer( 29508, "Andellion", 4, 3, 34.1, 27.4, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer(   914, "Ander Germaine", 2, 26, 79.4, 69.0, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  3607, "Androl Oakhand", 1, 22, 53.9, 90.0, "Darnassus", 1 ) -- zone: Teldrassil
	AddTrainer(  4215, "Anishar", 1, 7, 42.2, 36.8, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 16588, "Apothecary Antonivich", 3, 2, 52.4, 36.5, "Thrallmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 27023, "Apothecary Bressa", 4, 4, 36.2, 48.7, "The Hand of Vengeance", 2 ) -- zone: Dragonblight
	AddTrainer( 27029, "Apothecary Wormwick", 4, 4, 76.9, 62.2, "The Hand of Vengeance", 2 ) -- zone: Dragonblight
	AddTrainer( 15400, "Arathel Sunforge", 2, 11, 59.6, 62.6, "Silvermoon City", 2 ) -- zone: Eversong Woods
	AddTrainer( 16161, "Arcanist Sheynathren", 2, 11, 38.2, 72.5, "Silvermoon City", 2 ) -- zone: Eversong Woods
	AddTrainer( 29156, "Archmage Celindra", 4, 3, 56.5, 47.1, "Kirin Tor", 1 ) -- zone: Dalaran
	AddTrainer(  3047, "Archmage Shymm", 1, 25, 23.2, 13.4, "Undercity", 2 ) -- zone: Thunder Bluff
	AddTrainer( 18991, "Aresella", 3, 2, 26.3, 62.0, "Silvermoon City", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 26988, "Argo Strongstout", 4, 1, 57.2, 66.6, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer(  4087, "Arias'ta Bladesinger", 1, 7, 56.5, 46.3, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  4573, "Armand Cromwell", 2, 35, 80.8, 31.1, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  3373, "Arnok", 1, 17, 34.0, 84.5, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  5493, "Arnold Leland", 2, 26, 54.9, 69.4, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 33603, "Arthur Denny", 4, 8, 71.8, 20.9, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer( 26975, "Arthur Henslowe", 4, 1, 41.8, 54.3, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer(  4588, "Arthur Moore", 2, 35, 70.2, 58.1, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  5491, "Arthur the Faithful", 2, 26, 52.8, 45.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 17222, "Artificer Daelo", 1, 4, 48.0, 51.0, "Exodar", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 46709, "Arugi", 1, 17, 56.4, 61.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 19540, "Asarnan", 3, 4, 44.2, 33.7, "The Consortium", 3 ) -- zone: Netherstorm
	AddTrainer(  3026, "Aska Mistrunner", 1, 25, 51.1, 52.9, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 26996, "Awan Iceborn", 4, 1, 76.3, 37.0, "The Taunka", 2 ) -- zone: Borean Tundra
	AddTrainer( 29631, "Awilo Lon'gomba", 4, 3, 70.0, 38.4, "The Sunreavers", 2 ) -- zone: Dalaran
	AddTrainer(  6291, "Balthus Stoneflayer", 2, 14, 39.8, 32.5, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 33631, "Barien", 3, 6, 43.2, 64.7, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 18754, "Barim Spilthoof", 3, 2, 56.2, 38.6, "Orgrimmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 17514, "Bati", 1, 23, 46.5, 63.1, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 18988, "Baxter", 3, 2, 56.8, 37.5, "Thrallmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 17120, "Behomat", 1, 23, 55.4, 81.9, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 16663, "Belil", 2, 24, 79.1, 42.9, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer( 47571, "Belloc Brightblade", 1, 17, 49.1, 70.6, "Silvermoon City", 2 ) -- zone: Orgrimmar
	AddTrainer( 16669, "Bemarrin", 2, 24, 79.5, 39.0, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  3009, "Bena Winterhoof", 1, 25, 46.8, 33.5, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  4258, "Bengus Deepforge", 2, 14, 52.6, 41.4, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 26914, "Benjamin Clegg", 4, 6, 58.6, 62.8, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer(  3032, "Beram Skychaser", 1, 25, 22.2, 19.3, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 26911, "Bernadette Dexter", 4, 6, 59.9, 63.6, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer( 33587, "Bethany Cromwell", 4, 8, 72.4, 20.9, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer(  5114, "Bilban Tosslespanner", 2, 14, 66.2, 88.5, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer(  7954, "Binjy Featherwhistle", 2, 7, 56.3, 46.5, "Gnomeregan", 1 ) -- zone: Dun Morogh
	AddTrainer(  5144, "Bink", 2, 14, 27.0, 8.2, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer( 33586, "Binkie Brightgear", 4, 8, 72.2, 20.9, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer( 42331, "Bipsi Frostflinger", 2, 7, 34.5, 35.1, "Gnomeregan", 1 ) -- zone: Dun Morogh
	AddTrainer(  5957, "Birgitte Cranston", 1, 25, 22.4, 16.2, "Undercity", 2 ) -- zone: Thunder Bluff
	AddTrainer( 17245, "Blacksmith Calypso", 1, 4, 46.4, 71.1, "Ironforge", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 33609, "Blacksmithing", 3, 6, 44.1, 90.6, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 47575, "Boduro the Seeker", 3, 6, 62.4, 70.4, "Lower City", 3 ) -- zone: Shattrath City
	AddTrainer( 44393, "Bolner Hammerbeak", 2, 26, 66.0, 31.1, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 26959, "Booker Kells", 4, 6, 79.4, 29.3, "The Hand of Vengeance", 2 ) -- zone: Howling Fjord
	AddTrainer( 11178, "Borgosh Corebender", 1, 17, 75.9, 33.7, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  7232, "Borgus Steelhand", 2, 26, 59.8, 34.8, "Ironforge", 1 ) -- zone: Stormwind City
	AddTrainer( 26564, "Borus Ironbender", 4, 4, 36.6, 47.1, "Warsong Offensive", 2 ) -- zone: Dragonblight
	AddTrainer( 33639, "Botanist Alaenra", 3, 6, 38.5, 71.1, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 16644, "Botanist Nathera", 2, 24, 67.4, 18.6, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  3523, "Bowen Brisboise", 2, 31, 52.6, 55.6, "Undercity", 2 ) -- zone: Tirisfal Glades
	AddTrainer(  5142, "Braenna Flintcrag", 2, 14, 24.6, 8.8, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 43769, "Bralla Cloudwing", 2, 26, 70.2, 73.6, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 29924, "Brandig", 4, 10, 28.9, 74.9, nil, 1 ) -- zone: The Storm Peaks
	AddTrainer(  5149, "Brandur Ironhammer", 2, 14, 23.5, 6.1, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer(  1385, "Brawn", 2, 27, 31.7, 28.8, "nil", 2 ) -- zone: Stranglethorn Vale
	AddTrainer(  3001, "Brek Stonehoof", 1, 25, 34.6, 57.6, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  5172, "Briarthorn", 2, 14, 50.2, 5.6, "Gnomeregan", 1 ) -- zone: Ironforge
--	AddTrainer(  2836, "Brikk Keencraft", 2, 27, 29.0, 75.5, "Booty Bay", 3 ) -- zone: Stranglethorn Vale
	AddTrainer(  1681, "Brock Stoneseeker", 2, 17, 37.1, 47.8, "Ironforge", 1 ) -- zone: Loch Modan
	AddTrainer( 26905, "Brom Brewbaster", 4, 6, 58.2, 62.1, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer(  4598, "Brom Killian", 2, 35, 55.8, 37.0, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  1702, "Bronk Guzzlegear", 2, 7, 57.3, 48.7, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  5484, "Brother Benjamin", 2, 26, 50.7, 44.7, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  5489, "Brother Joshua", 2, 26, 49.6, 45.0, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 50022, "Brulla", 1, 16, 49.5, 60.8, "Orgrimmar", 2 ) -- zone: Northern Barrens
	AddTrainer( 18771, "Brumman", 3, 2, 54.1, 64.0, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer(  7869, "Brumn Winterhoof", 2, 2, 28.2, 45.0, "Thunder Bluff", 2 ) -- zone: Arathi Highlands
	AddTrainer( 26976, "Brunna Ironaxe", 4, 1, 42.6, 53.2, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer( 26992, "Brynna Wilson", 4, 1, 57.9, 66.3, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 18990, "Burko", 3, 2, 22.4, 39.3, "Exodar", 1 ) -- zone: Hellfire Peninsula
	AddTrainer(  8736, "Buzzek Bracketswing", 1, 21, 52.3, 27.7, "Gadgetzan", 3 ) -- zone: Tanaris
	AddTrainer(  6094, "Byancie", 1, 22, 55.0, 49.6, "Darnassus", 1 ) -- zone: Teldrassil
	AddTrainer( 16642, "Camberon", 2, 24, 66.1, 17.4, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  2132, "Carolai Anise", 2, 31, 59.5, 52.2, "Undercity", 2 ) -- zone: Tirisfal Glades
	AddTrainer( 26960, "Carter Tiffens", 4, 6, 79.3, 28.8, "The Hand of Vengeance", 2 ) -- zone: Howling Fjord
--	AddTrainer(  7870, "Caryssia Moonhunter", 1, 12, 89.4, 46.5, nil, 1 ) -- zone: Feralas
	AddTrainer( 30713, "Catarina Stanford", 2, 26, 49.7, 74.8, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 44395, "Celestine of the Harvest", 2, 26, 57.6, 24.9, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 19369, "Celie Steelwing", 3, 5, 37.2, 58.5, nil, 1 ) -- zone: Shadowmoon Valley
	AddTrainer( 28699, "Charles Worth", 4, 3, 36.4, 33.7, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer(  3069, "Chaw Stronghide", 1, 15, 45.5, 57.9, "Thunder Bluff", 2 ) -- zone: Mulgore
	AddTrainer( 25277, "Chief Engineer Leveny", 4, 1, 42.6, 53.7, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer(  3136, "Clarise Gnarltree", 2, 8, 74.0, 48.6, "Stormwind", 1 ) -- zone: Duskwood
	AddTrainer(  5690, "Clyde Kellen", 2, 31, 67.0, 51.0, "Undercity", 2 ) -- zone: Tirisfal Glades
	AddTrainer( 50011, "Cona", 1, 9, 56.2, 75.2, "Darkspear Trolls", 2 ) -- zone: Durotar
	AddTrainer(  1355, "Cook Ghilm", 2, 7, 75.6, 52.9, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer( 33619, "Cooking", 3, 6, 43.3, 91.4, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 26981, "Crog Steelspine", 4, 1, 40.8, 55.3, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer(  3087, "Crystal Boughman", 2, 19, 22.8, 40.6, "Stormwind", 1 ) -- zone: Redridge Mountains
	AddTrainer( 33588, "Crystal Brightspark", 4, 8, 71.6, 21.0, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer(  3603, "Cyndra Kindwhisper", 1, 22, 57.0, 53.0, "Darnassus", 1 ) -- zone: Teldrassil
	AddTrainer( 17215, "Daedal", 1, 4, 48.5, 51.5, "Exodar", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 33635, "Daenril", 3, 6, 41.6, 63.4, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 44394, "Dalga Hammerbeak", 2, 26, 66.1, 31.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 18749, "Dalinna", 3, 2, 56.6, 37.1, "Thrallmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer(  1701, "Dank Drizzlecut", 2, 7, 76.5, 53.7, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  4211, "Dannelor", 1, 7, 51.6, 30.7, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 16667, "Danwe", 2, 24, 76.5, 40.9, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer( 50729, "Darba the Crone", 2, 14, 50.0, 5.9, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 27001, "Darin Goodstitch", 4, 1, 57.5, 72.4, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 47579, "Dariness the Learned", 4, 3, 48.3, 38.4, "Kirin Tor", 1 ) -- zone: Dalaran
	AddTrainer( 45339, "Dark Cleric Cecille", 1, 17, 48.2, 73.0, "Undercity", 2 ) -- zone: Orgrimmar
	AddTrainer( 43693, "Darlene Stokx", 2, 26, 77.5, 67.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 19187, "Darmari", 3, 6, 66.8, 67.1, "Lower City", 3 ) -- zone: Shattrath City
	AddTrainer(  7315, "Darnath Bladesinger", 1, 7, 56.5, 46.3, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  5159, "Daryl Riknussun", 2, 14, 60.2, 36.3, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  2399, "Daryl Stack", 2, 13, 58.0, 47.8, "Undercity", 2 ) -- zone: Hillsbrad Foothills
	AddTrainer(  3290, "Deek Fizzlebizz", 2, 17, 42.8, 10.0, "Gnomeregan", 1 ) -- zone: Loch Modan
	AddTrainer(  5885, "Deino", 1, 17, 38.5, 85.9, "Orgrimmar", 2 ) -- zone: Orgrimmar
--	AddTrainer(  6299, "Delfrum Flintbeard", 1, 6, 38.2, 41.0, "nil", 1 ) -- zone: Darkshore
	AddTrainer(   461, "Demisette Cloyce", 2, 26, 39.3, 84.9, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 23103, "Derek the Undying", 2, 35, 56.6, 17.8, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 28696, "Derik Marks", 4, 3, 35.2, 28.4, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 28700, "Diane Cannings", 4, 3, 35.0, 28.5, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 29513, "Didi the Wrench", 4, 3, 39.5, 25.0, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer(  7312, "Dink", 2, 14, 27.0, 8.2, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer( 36615, "Doc Zapnozzle", 5, 3, 42.7, 16.3, "Bilgewater Cartel", 2 ) -- zone: The Lost Isles
	AddTrainer( 12920, "Doctor Gregory Victor", 2, 2, 68.4, 37.8, "Undercity", 2 ) -- zone: Arathi Highlands
	AddTrainer( 12939, "Doctor Gustaf VanHowzen", 1, 10, 67.6, 48.8, nil, 1 ) -- zone: Dustwallow Marsh
	AddTrainer(  4611, "Doctor Herbert Halsey", 2, 35, 47.7, 73.0, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 28704, "Dorothy Egan", 4, 3, 43.1, 34.3, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 11097, "Drakk Stonehand", 2, 30, 13.4, 43.4, nil, 1 ) -- zone: The Hinterlands
	AddTrainer( 16780, "Drathen", 2, 24, 76.6, 68.6, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer( 33683, "Dremm", 3, 6, 37.3, 26.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 42324, "Drill Sergeant Steamcrank", 2, 7, 34.3, 37.4, "Gnomeregan", 1 ) -- zone: Dun Morogh
	AddTrainer( 17488, "Dulvi", 1, 4, 48.9, 51.1, "Exodar", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 33580, "Dustin Vail", 4, 8, 73.0, 20.8, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer(  3174, "Dwukk", 1, 9, 52.0, 40.8, "Orgrimmar", 2 ) -- zone: Durotar
	AddTrainer( 50714, "Dyrhara", 1, 7, 37.5, 81.1, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 16749, "Edirah", 1, 23, 47.2, 61.9, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  1103, "Eldrin", 2, 10, 79.3, 69.0, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 30717, "Elise Brightletter", 2, 14, 60.7, 45.2, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  4165, "Elissa Dumas", 1, 7, 43.9, 78.4, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 26906, "Elizabeth Jackson", 4, 6, 58.6, 62.8, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer(  5498, "Elsharin", 2, 26, 48.4, 87.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 26954, "Emil Autumn", 4, 6, 78.7, 28.3, "The Hand of Vengeance", 2 ) -- zone: Howling Fjord
	AddTrainer( 28693, "Enchanter Nalthanis", 4, 3, 38.9, 39.9, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 33610, "Enchanting", 3, 6, 43.9, 90.4, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 33633, "Enchantress Andiala", 3, 6, 55.8, 74.6, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 19251, "Enchantress Volali", 3, 6, 43.2, 92.3, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 33634, "Engineer Sinbei", 3, 6, 43.6, 65.0, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 33611, "Engineering", 3, 6, 43.6, 90.4, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 45718, "Environmental Engineer Linza", 1, 17, 36.4, 85.0, "Bilgewater Cartel", 2 ) -- zone: Orgrimmar
--	AddTrainer(  5883, "Enyo", 1, 17, 38.7, 85.2, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 26980, "Eorain Dawnstrike", 4, 1, 41.2, 53.9, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer( 17487, "Erin Kelly", 1, 4, 46.2, 70.5, "Stormwind", 1 ) -- zone: Azuremyst Isle
	AddTrainer(  4214, "Erion Shadewhisper", 1, 7, 40.0, 39.7, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  4552, "Eunice Burch", 2, 35, 62.4, 44.8, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 33583, "Fael Morningsong", 4, 8, 73.0, 20.6, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer( 43428, "Faeyrin Willowmoon", 1, 6, 50.7, 20.8, "Darnassus", 1 ) -- zone: Darkshore
	AddTrainer( 47576, "Falda Fardelve", 4, 1, 57.6, 71.7, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 26987, "Falorn Nightwhisper", 4, 1, 57.8, 71.9, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 19778, "Farii", 1, 23, 45.0, 24.0, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 23127, "Farseer Javad", 2, 14, 55.3, 29.1, "Exodar", 1 ) -- zone: Ironforge
	AddTrainer( 50174, "Farseer Lopaa", 4, 3, 38.1, 65.1, "Exodar", 1 ) -- zone: Dalaran
	AddTrainer( 20407, "Farseer Umbrua", 2, 26, 65.8, 31.0, "Exodar", 1 ) -- zone: Stormwind City
	AddTrainer( 18753, "Felannia", 3, 2, 52.3, 36.1, "Thrallmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 26999, "Fendrig Redbeard", 4, 1, 57.5, 66.2, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer(  5167, "Fenthwick", 2, 14, 51.8, 15.3, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer( 19478, "Fera Palerunner", 3, 1, 53.7, 55.0, nil, 2 ) -- zone: Blade's Edge Mountains
	AddTrainer( 30715, "Feyden Darkin", 1, 7, 56.8, 31.0, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  5127, "Fimble Finespindle", 2, 14, 40.1, 33.5, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer(  1676, "Finbus Geargrind", 2, 8, 77.4, 48.7, "Gnomeregan", 1 ) -- zone: Duskwood
	AddTrainer( 29514, "Findle Whistlesteam", 4, 3, 39.3, 24.9, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 33621, "First Aid", 3, 6, 43.6, 91.6, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 33623, "Fishing", 3, 6, 43.4, 91.4, "The Scryers", 2 ) -- zone: Shattrath City
--	AddTrainer(   908, "Flora Silverwind", 2, 27, 42.0, 74.6, "Booty Bay", 2 ) -- zone: Stranglethorn Vale
	AddTrainer( 33682, "Fono", 3, 6, 36.1, 48.1, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 11031, "Franklin Lloyd", 2, 35, 75.9, 73.7, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  3181, "Fremal Doohickey", 2, 39, 10.5, 60.2, "Gnomeregan", 1 ) -- zone: Wetlands
	AddTrainer( 18987, "Gaston", 3, 2, 54.1, 63.5, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer( 26982, "Geba'li", 4, 1, 41.6, 53.4, "Warsong Offensive", 2 ) -- zone: Borean Tundra
	AddTrainer(  5513, "Gelman Stonehand", 2, 26, 59.6, 37.5, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  4254, "Geofram Bouldertoe", 2, 14, 50.1, 26.2, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  1346, "Georgio Bolero", 2, 26, 53.3, 81.5, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  1470, "Ghak Healtouch", 2, 17, 37.0, 49.2, "Ironforge", 1 ) -- zone: Loch Modan
	AddTrainer(  5157, "Gimble Thistlefuzz", 2, 14, 60.4, 45.5, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer(  3345, "Godan", 1, 17, 53.3, 49.4, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 46357, "Gonto", 1, 17, 44.7, 78.4, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  1229, "Granis Swiftaxe", 2, 7, 54.5, 50.9, "Ironforge", 1 ) -- zone: Dun Morogh
--	AddTrainer(  2627, "Grarnik Goodstitch", 2, 27, 28.7, 76.8, "Booty Bay", 3 ) -- zone: Stranglethorn Vale
	AddTrainer(  1699, "Gremlock Pilsnor", 2, 7, 54.6, 50.6, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer( 49781, "Grendin Swiftaxe", 2, 17, 34.0, 46.7, "Ironforge", 1 ) -- zone: Loch Modan
	AddTrainer(  3353, "Grezz Ragefist", 1, 17, 73.6, 45.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 21087, "Grikka", 3, 1, 76.8, 65.5, nil, 2 ) -- zone: Blade's Edge Mountains
	AddTrainer(  5161, "Grimnur Stonebrand", 2, 14, 48.2, 6.7, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer(  4193, "Grondal Moonbreeze", 1, 6, 38.2, 40.5, "nil", 1 ) -- zone: Darkshore (Dead in Cata)
	AddTrainer( 26912, "Grumbol Stoutpick", 4, 6, 59.9, 63.9, "Valiance Expedition", 1 ) -- zone: Howling Fjord
	AddTrainer(  5164, "Grumnus Steelshaper", 2, 14, 50.5, 42.7, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 19341, "Grutah", 3, 5, 29.7, 31.5, nil, 2 ) -- zone: Shadowmoon Valley
	AddTrainer(  3557, "Guillaume Sorouy", 2, 25, 43.2, 41.0, "Undercity", 2 ) -- zone: Silverpine Forest
	AddTrainer( 26961, "Gunter Hansen", 4, 6, 78.3, 28.2, "The Hand of Vengeance", 2 ) -- zone: Howling Fjord
	AddTrainer( 50152, "Hahna Moonscrest", 4, 3, 58.3, 27.2, "Thunder Bluff", 2 ) -- zone: Dalaran
	AddTrainer( 11098, "Hahrana Ironhide", 1, 12, 74.4, 43.1, "Thunder Bluff", 2 ) -- zone: Feralas
	AddTrainer( 18772, "Hama", 3, 2, 54.1, 63.6, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer( 19063, "Hamanar", 3, 6, 35.8, 20.6, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 47569, "Hammon the Jaded", 1, 7, 42.7, 83.3, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 33640, "Hanlir", 3, 6, 58.4, 75.2, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 17513, "Harnan", 1, 23, 46.8, 62.7, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  3179, "Harold Riggs", 2, 39, 8.1, 58.7, "Stormwind", 1 ) -- zone: Wetlands
	AddTrainer( 44238, "Harrison Jones", 2, 26, 85.7, 26.0, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 33616, "Herbalism", 3, 6, 43.9, 91.1, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 11074, "Hgarth", 1, 20, 51.8, 59.8, "Orgrimmar", 2 ) -- zone: Stonetalon Mountains
	AddTrainer( 19252, "High Enchanter Bardolan", 3, 6, 43.2, 92.2, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 11406, "High Priest Rohan", 2, 14, 25.1, 8.4, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(   376, "High Priestess Laurena", 2, 26, 49.5, 44.6, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 31084, "Highlord Darion Mograine", 2, 9, 83.5, 49.5, "Knights of the Ebon Blade", 3 ) -- zone: Eastern Plaguelands
	AddTrainer( 31238, "Hira Snowdawn", 4, 3, 70.0, 46.0, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 44783, "Hiwahi Three-Feathers", 1, 17, 38.9, 50.2, "Thunder Bluff", 2 ) -- zone: Orgrimmar
	AddTrainer(  5165, "Hulfdan Blackbeard", 2, 14, 51.7, 14.5, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 16823, "Humphry", 3, 2, 56.8, 63.8, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer( 18779, "Hurnak Grimmord", 3, 2, 56.7, 63.8, "Honor Hold", 1 ) -- zone: Hellfire Peninsula
	AddTrainer(  5480, "Ilsa Corbin", 2, 26, 80.0, 69.8, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 29505, "Imindril Spearsong", 4, 3, 45.2, 28.7, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 16653, "Inethven", 2, 24, 59.0, 19.6, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer( 33615, "Inscription", 3, 6, 43.5, 90.7, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 20791, "Iorioa", 3, 6, 59.1, 47.2, "The Sha'tar", 1 ) -- zone: Shattrath City
	AddTrainer( 33641, "Irduil", 3, 6, 40.9, 63.3, "The Scryers", 2 ) -- zone: Shattrath City
	AddTrainer( 11146, "Ironus Coldsteel", 2, 14, 50.4, 43.3, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 19185, "Jack Trapper", 3, 6, 62.8, 67.9, "nil", 3 ) -- zone: Shattrath City
--	AddTrainer(  9584, "Jalane Ayrole", 2, 26, 40.6, 83.9, "nil", 1 ) -- zone: Stormwind City
	AddTrainer(  4596, "James Van Brunt", 2, 35, 61.2, 29.9, "Undercity", 2 ) -- zone: Undercity
--	AddTrainer( 26955, "Jamesina Watterly", 4, 6, 78.5, 30.0, "nil", 2 ) -- zone: Howling Fjord
	AddTrainer(  3404, "Jandi", 1, 17, 34.7, 63.1, "Darkspear Trolls", 2 ) -- zone: Orgrimmar
	AddTrainer(  8142, "Jannos Lighthoof", 1, 12, 76.0, 42.2, "Thunder Bluff", 2 ) -- zone: Feralas
	AddTrainer(  4753, "Jartsam", 1, 7, 42.6, 32.9, "Darnassus", 1 ) -- zone: Darnassus
--	AddTrainer(  2837, "Jaxin Chong", 2, 27, 28.1, 78.0, "nil", 3 ) -- zone: Stranglethorn Vale
--	AddTrainer( 19539, "Jazdalaad", 3, 4, 44.5, 34.0, "nil", 3 ) -- zone: Netherstorm
	AddTrainer( 28698, "Jedidiah Handers", 4, 3, 41.4, 25.9, "Kirin Tor", 3 ) -- zone: Dalaran
--	AddTrainer( 11037, "Jenna Lemkenilli", 1, 6, 38.3, 41.1, "nil", 1 ) -- zone: Darkshore
	AddTrainer(  5497, "Jennea Cannon", 2, 26, 49.8, 85.9, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 28958, "Jessa Weaver", 4, 3, 55.1, 47.1, "Kirin Tor", 1 ) -- zone: Dalaran
	AddTrainer( 33614, "Jewelcrafting", 3, 6, 43.5, 90.7, "The Scryers", 3 ) -- zone: Shattrath City
--	AddTrainer( 14743, "Jhordy Lapforge", 1, 21, 52.3, 26.9, "nil", 3 ) -- zone: Tanaris
	AddTrainer( 33678, "Jijia", 3, 6, 38.1, 30.3, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 30706, "Jo'mah", 1, 17, 56.2, 46.5, "Darkspear Trolls", 2 ) -- zone: Orgrimmar
--	AddTrainer( 18773, "Johan Barnes", 3, 2, 53.7, 66.1, "nil", 1 ) -- zone: Hellfire Peninsula
--	AddTrainer(  3555, "Johan Focht", 2, 25, 43.4, 40.5, "nil", 2 ) -- zone: Silverpine Forest
--	AddTrainer( 25099, "Jonathan Garrett", 3, 5, 29.2, 28.5, "nil", 2 ) -- zone: Shadowmoon Valley
--	AddTrainer( 26962, "Jonathan Lewis", 4, 6, 79.3, 29.0, "nil", 2 ) -- zone: Howling Fjord
	AddTrainer(  5153, "Jormund Stonebrow", 2, 14, 43.2, 29.2, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  4576, "Josef Gregorian", 2, 35, 70.7, 30.3, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 33589, "Joseph Wilson", 4, 8, 71.4, 22.4, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer(  4578, "Josephine Lister", 2, 35, 86.5, 22.3, "Undercity", 2 ) -- zone: Undercity
--	AddTrainer( 27034, "Josric Fame", 4, 4, 75.9, 63.2, "nil", 2 ) -- zone: Dragonblight
	AddTrainer(  5145, "Juli Stormkettle", 2, 14, 26.2, 6.8, "Gnomeregan", 1 ) -- zone: Ironforge
--	AddTrainer( 17634, "K. Lee Smallfry", 3, 8, 68.6, 50.2, "nil", 1 ) -- zone: Zangarmarsh
--	AddTrainer( 21493, "Kablamm Farflinger", 3, 4, 32.9, 63.7, "nil", 3 ) -- zone: Netherstorm
	AddTrainer(  4566, "Kaelystia Hatebringer", 2, 35, 85.6, 14.2, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  3028, "Kah Mistrunner", 1, 25, 56.6, 47.6, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
--	AddTrainer( 18751, "Kalaen", 3, 2, 56.8, 37.7, "nil", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 19775, "Kalinda", 2, 24, 90.5, 74.1, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer( 16272, "Kanaria", 2, 11, 48.5, 47.6, "nil", 2 ) -- zone: Eversong Woods
	AddTrainer(  3344, "Kardris Dreamseeker", 1, 17, 39.0, 47.4, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 45548, "Kark Helmbreaker", 1, 17, 36.0, 82.5, "Thunder Bluff", 2 ) -- zone: Orgrimmar
--	AddTrainer(  2998, "Karn Stonehoof", 1, 25, 39.0, 56.5, "nil", 2 ) -- zone: Thunder Bluff
	AddTrainer(  3365, "Karolek", 1, 17, 61.0, 54.8, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 28705, "Katherine Lee", 4, 3, 40.8, 65.6, "The Silver Covenant", 1 ) -- zone: Dalaran
	AddTrainer(  5492, "Katherine the Pure", 2, 26, 52.7, 44.9, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 17504, "Kazi", 1, 23, 56.8, 84.1, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 16640, "Keelen Sheets", 2, 24, 57.0, 50.1, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer(  7231, "Kelgruk Bloodaxe", 1, 17, 81.9, 18.3, "nil", 2 ) -- zone: Orgrimmar
	AddTrainer(  1901, "Kelstrum Stonebreaker", 2, 14, 66.8, 90.0, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  5113, "Kelv Sternhammer", 2, 14, 70.1, 91.1, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer( 49939, "Kenral Nightwind", 1, 6, 50.7, 18.6, "Darnassus", 1 ) -- zone: Darkshore
	AddTrainer(  4752, "Kildar", 1, 17, 61.2, 34.7, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 17505, "Killac", 1, 23, 46.3, 88.8, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  7087, "Killian Hagey", 2, 35, 70.3, 59.9, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 26994, "Kirea Moondancer", 4, 1, 57.8, 71.8, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 33637, "Kirembri Silvermane", 3, 6, 58.0, 75.0, "The Scryers", 2 ) -- zone: Shattrath City
--	AddTrainer( 11072, "Kitta Firewind", 2, 10, 64.9, 70.6, "nil", 1 ) -- zone: Elwynn Forest
	AddTrainer(  3013, "Komin Winterhoof", 1, 25, 49.8, 39.8, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 33681, "Korim", 3, 6, 37.5, 27.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 20124, "Kradu Grimblade", 3, 6, 69.6, 43.6, "Lower City", 3 ) -- zone: Shattrath City
	AddTrainer( 45540, "Krenk Choplimb", 1, 17, 37.1, 87.2, "Bilgewater Cartel", 2 ) -- zone: Orgrimmar
--	AddTrainer( 26952, "Kristen Smythe", 4, 6, 79.2, 29.0, "nil", 2 ) -- zone: Howling Fjord
--	AddTrainer( 18747, "Krugosh", 3, 2, 55.5, 37.6, "nil", 2 ) -- zone: Hellfire Peninsula
--	AddTrainer(  3175, "Krunn", 1, 9, 51.9, 40.9, "nil", 2 ) -- zone: Durotar
--	AddTrainer( 45286, "KTC Train-a-Tron Deluxe", 5, 3, 53.2, 35.7, "nil", 2 ) -- zone: The Lost Isles
	AddTrainer( 33581, "Kul'de", 4, 8, 71.8, 20.8, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer(  8144, "Kulleg Stonehorn", 1, 12, 74.4, 43.0, "Thunder Bluff", 2 ) -- zone: Feralas
--	AddTrainer(  6297, "Kurdram Stonehammer", 1, 6, 38.2, 41.1, "nil", 1 ) -- zone: Darkshore
--	AddTrainer(  7948, "Kylanna Windwhisper", 1, 12, 32.6, 43.8, "nil", 1 ) -- zone: Feralas
--	AddTrainer(  3964, "Kylanna", 1, 2, 50.8, 67.1, "nil", 1 ) -- zone: Ashenvale
--	AddTrainer( 19186, "Kylene", 3, 6, 76.5, 33.0, "nil", 3 ) -- zone: Shattrath City
	AddTrainer(  3036, "Kym Wildmane", 1, 25, 76.4, 31.4, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 29195, "Lady Alistra", 2, 9, 83.7, 44.6, "Knights of the Ebon Blade", 3 ) -- zone: Eastern Plaguelands
--	AddTrainer( 26903, "Lanolis Dewdrop", 4, 6, 58.4, 62.3, "nil", 1 ) -- zone: Howling Fjord
	AddTrainer(  2485, "Larimaine Purdue", 2, 26, 50.2, 86.1, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 50732, "Larn Caverndeep", 2, 14, 49.9, 6.3, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  5941, "Lau'Tiki", 1, 9, 57.3, 77.1, "Darkspear Trolls", 2 ) -- zone: Durotar
	AddTrainer(  4616, "Lavinia Crowe", 2, 35, 61.9, 60.9, "Undercity", 2 ) -- zone: Undercity
	AddTrainer( 33612, "Leatherworking", 3, 6, 43.7, 90.8, "The Scryers", 3 ) -- zone: Shattrath City
--	AddTrainer( 18775, "Lebowski", 3, 2, 55.7, 65.5, "nil", 1 ) -- zone: Hellfire Peninsula
	AddTrainer(  2492, "Lexington Mortaim", 2, 35, 84.6, 16.8, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  5518, "Lilliam Sparkspindle", 2, 26, 62.9, 32.2, "Gnomeregan", 1 ) -- zone: Stormwind City
--	AddTrainer(  5499, "Lilyssia Nightbreeze", 2, 26, 55.6, 85.9, "nil", 1 ) -- zone: Stormwind City
	AddTrainer( 28703, "Linzy Blackbolt", 4, 3, 42.4, 32.2, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer(   911, "Llane Beshere", 2, 10, 50.2, 42.2, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 12042, "Loganaar", 1, 13, 52.6, 40.6, "Cenarion Circle", 3 ) -- zone: Moonglade
	AddTrainer(   928, "Lord Grayson Shadowbreaker", 2, 26, 52.7, 45.0, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 29196, "Lord Thorval", 2, 9, 80.9, 43.9, "Knights of the Ebon Blade", 3 ) -- zone: Eastern Plaguelands
	AddTrainer( 13283, "Lord Tony Romano", 2, 26, 79.6, 61.4, "Stormwind", 1 ) -- zone: Stormwind City
--	AddTrainer( 19052, "Lorokeem", 3, 6, 45.4, 19.5, "nil", 3 ) -- zone: Shattrath City
	AddTrainer(  1317, "Lucan Cordell", 2, 26, 53.1, 74.2, "Stormwind", 1 ) -- zone: Stormwind City
--	AddTrainer( 16723, "Lucc", 1, 23, 27.5, 60.9, "nil", 1 ) -- zone: The Exodar
	AddTrainer( 46675, "Lugrah", 1, 17, 72.4, 34.5, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  3332, "Lumak", 1, 17, 66.6, 41.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 16755, "Lunaraa", 1, 23, 46.2, 63.0, "Exodar", 1 ) -- zone: The Exodar
--	AddTrainer( 16688, "Lynalis", 2, 24, 84.0, 80.2, "nil", 2 ) -- zone: Silvermoon City
	AddTrainer(   913, "Lyria Du Lac", 2, 10, 41.1, 65.9, "Stormwind", 1 ) -- zone: Elwynn Forest
--	AddTrainer( 17637, "Mack Diver", 3, 8, 33.9, 51.0, "nil", 2 ) -- zone: Zangarmarsh
	AddTrainer( 50715, "Maelir", 1, 7, 37.7, 81.1, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  3363, "Magar", 1, 17, 60.6, 59.0, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(   331, "Maginor Dumas", 2, 26, 49.0, 87.6, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  1228, "Magis Sparkmantle", 2, 7, 54.6, 50.4, "Gnomeregan", 1 ) -- zone: Dun Morogh
--	AddTrainer( 16160, "Magistrix Eredania", 2, 11, 38.2, 72.6, "nil", 2 ) -- zone: Eversong Woods
	AddTrainer( 35133, "Maigra Keenfeather", 4, 1, 58.9, 68.2, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer(  3357, "Makaru", 1, 17, 72.3, 35.0, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 12025, "Malvor", 1, 13, 45.5, 47.0, "Cenarion Circle", 1 ) -- zone: Moonglade
	AddTrainer( 29507, "Manfred Staller", 4, 3, 34.2, 29.5, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 28742, "Marcia Chase", 4, 3, 52.6, 64.7, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 30711, "Margaux Parchley", 2, 35, 61.1, 58.6, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  1292, "Maris Granger", 2, 26, 72.1, 62.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  4614, "Martha Alliestar", 2, 35, 54.3, 49.7, "Undercity", 2 ) -- zone: Undercity
	AddTrainer(  4591, "Mary Edras", 2, 35, 73.4, 55.8, "Undercity", 2 ) -- zone: Undercity
--	AddTrainer( 16253, "Master Chef Mouldier", 2, 12, 48.3, 30.9, "nil", 2 ) -- zone: Ghostlands
	AddTrainer( 23128, "Master Pyreanor", 1, 17, 49.2, 71.3, "Silvermoon City", 2 ) -- zone: Orgrimmar
--	AddTrainer(  3137, "Matt Johnson", 2, 8, 74.0, 49.7, "nil", 1 ) -- zone: Duskwood
	AddTrainer( 49900, "Maxx Avalanche", 1, 3, 56.7, 49.6, "Bilgewater Cartel", 2 ) -- zone: Azshara
	AddTrainer( 44919, "Maztha", 1, 17, 49.1, 59.2, "Orgrimmar", 2 ) -- zone: Orgrimmar
--	AddTrainer(  4159, "Me'lynn", 1, 7, 61.7, 23.0, "nil", 1 ) -- zone: Darnassus
--	AddTrainer( 11557, "Meilosh", 1, 11, 65.7, 2.9, "nil", 2 ) -- zone: Felwood
	AddTrainer( 16724, "Miall", 1, 23, 60.5, 90.0, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer(  3184, "Miao'zan", 1, 9, 55.5, 74.0, "Darkspear Trolls", 2 ) -- zone: Durotar
--	AddTrainer( 30721, "Michael Schwan", 3, 2, 53.9, 65.5, "nil", 1 ) -- zone: Hellfire Peninsula
	AddTrainer(  2329, "Michelle Belle", 2, 10, 43.4, 65.6, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 19184, "Mildred Fletcher", 3, 6, 66.6, 13.4, "Lower City", 3 ) -- zone: Shattrath City
	AddTrainer(  2489, "Milstaff Stormeye", 2, 14, 25.4, 7.2, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 26916, "Mindri Dinkles", 4, 6, 58.6, 62.8, "nil", 1 ) -- zone: Howling Fjord
	AddTrainer( 33617, "Mining", 3, 6, 43.7, 91.1, "The Scryers", 3 ) -- zone: Shattrath City
	AddTrainer( 33636, "Miralisse", 3, 6, 41.4, 63.5, "The Scryers", 3 ) -- zone: Shattrath City
--	AddTrainer( 17442, "Moordo", 1, 4, 44.8, 23.8, "nil", 1 ) -- zone: Azuremyst Isle
	AddTrainer( 18755, "Moorutu", 3, 2, 56.3, 38.5, "Thrallmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 16752, "Muaat", 1, 23, 59.9, 88.1, "Exodar", 1 ) -- zone: The Exodar
--	AddTrainer(  1382, "Mudduk", 2, 27, 31.4, 28.0, "nil", 2 ) -- zone: Stranglethorn Vale
--	AddTrainer( 11025, "Mukdrak", 1, 9, 52.2, 40.8, "nil", 2 ) -- zone: Durotar
	AddTrainer( 16719, "Mumman", 1, 23, 55.8, 27.0, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 46741, "Muraga", 1, 17, 54.9, 50.5, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 50034, "Musu Gentlemoon", 1, 16, 49.5, 61.3, "Thunder Bluff", 2 ) -- zone: Northern Barrens
--	AddTrainer(  2834, "Myizz Luckycatch", 2, 27, 0.0, 0.0, "Booty Bay", 2 ) -- zone: Stranglethorn Vale
--	AddTrainer(  3605, "Nadyia Maneweaver", 1, 22, 41.8, 49.5, "nil", 1 ) -- zone: Teldrassil
--	AddTrainer( 16725, "Nahogg", 1, 23, 40.5, 39.2, "nil", 1 ) -- zone: The Exodar
--	AddTrainer( 18993, "Naka", 3, 8, 78.5, 63.0, "nil", 3 ) -- zone: Zangarmarsh
--	AddTrainer( 29509, "Namha Moonwater", 4, 3, 36.3, 29.4, "nil", 3 ) -- zone: Dalaran
	AddTrainer( 16654, "Narinth", 2, 24, 58.6, 20.2, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer(  8153, "Narv Hidecrafter", 1, 8, 55.3, 56.3, "nil", 2 ) -- zone: Desolace
--	AddTrainer( 30722, "Neferatti", 3, 2, 52.3, 36.1, "nil", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 38242, "Nekali", 1, 9, 65.0, 84.2, "Darkspear Trolls", 2 ) -- zone: Durotar
	AddTrainer( 33680, "Nemiha", 3, 6, 36.2, 47.7, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 46716, "Nerog", 1, 17, 55.3, 55.7, "Orgrimmar", 2 ) -- zone: Orgrimmar
--	AddTrainer( 24868, "Niobe Whizzlespark", 3, 5, 36.7, 54.8, "nil", 1 ) -- zone: Shadowmoon Valley
	AddTrainer(  5150, "Nissa Firestone", 2, 14, 54.9, 58.4, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 45559, "Nivi Weavewell", 1, 17, 41.1, 79.7, "nil", 2 ) -- zone: Orgrimmar
--	AddTrainer(  8126, "Nixx Sprocketspring", 1, 21, 52.5, 27.3, "nil", 3 ) -- zone: Tanaris
--	AddTrainer( 29233, "Nurse Applewood", 4, 1, 41.7, 54.5, "nil", 2 ) -- zone: Borean Tundra
--	AddTrainer(  5759, "Nurse Neela", 2, 31, 61.8, 52.8, "nil", 2 ) -- zone: Tirisfal Glades
--	AddTrainer( 16731, "Nus", 1, 23, 39.0, 22.5, "nil", 1 ) -- zone: The Exodar
	AddTrainer( 16726, "Ockil", 1, 23, 53.9, 92.5, "Exodar", 1 ) -- zone: The Exodar
--	AddTrainer(  7406, "Oglethorpe Obnoticus", 2, 27, 28.3, 76.3, "nil", 3 ) -- zone: Stranglethorn Vale
	AddTrainer( 11177, "Okothos Ironrager", 1, 17, 75.4, 34.0, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 26993, "Old Man Robert", 4, 1, 57.9, 71.6, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 44975, "Old Umbehto", 1, 17, 35.0, 67.3, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 28706, "Olisarra the Kind", 4, 3, 36.8, 37.0, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 33590, "Oluros", 4, 8, 71.4, 20.8, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer( 33675, "Onodo", 3, 6, 37.6, 30.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 44781, "Opuno Ironhorn", 1, 17, 40.5, 49.9, "Thunder Bluff", 2 ) -- zone: Orgrimmar
	AddTrainer( 29506, "Orland Schaeffer", 4, 3, 44.7, 28.9, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer(  5166, "Ormyr Flinteye", 2, 14, 52.4, 15.0, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 26972, "Orn Tenderhoof", 4, 1, 42.0, 54.2, "nil", 2 ) -- zone: Borean Tundra
	AddTrainer(   918, "Osborne the Night Man", 2, 26, 77.5, 65.0, "Stormwind", 1 ) -- zone: Stormwind City
--	AddTrainer( 26915, "Ounhulo", 4, 6, 59.9, 63.8, "nil", 1 ) -- zone: Howling Fjord
	AddTrainer(  2798, "Pand Stonebinder", 1, 25, 29.4, 21.2, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 50158, "Pathstalker Ralsir", 4, 3, 38.3, 64.8, "The Silver Covenant", 1 ) -- zone: Dalaran
	AddTrainer(  1700, "Paxton Ganter", 2, 7, 42.5, 38.6, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  5882, "Pephredo", 1, 17, 38.5, 85.7, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 16280, "Perascamin", 2, 11, 61.2, 54.0, "Silvermoon City", 2 ) -- zone: Eversong Woods
	AddTrainer( 43431, "Periale", 1, 6, 51.4, 19.1, "Darnassus", 1 ) -- zone: Darkshore
--	AddTrainer(  7866, "Peter Galen", 1, 3, 37.5, 65.5, "nil", 1 ) -- zone: Azshara
	AddTrainer(  4567, "Pierce Shackleton", 2, 35, 85.6, 14.6, "Undercity", 2 ) -- zone: Undercity
--	AddTrainer(  8128, "Pikkle", 1, 21, 51.1, 28.1, "nil", 3 ) -- zone: Tanaris
	AddTrainer( 30709, "Poshken Hardbinder", 1, 25, 29.2, 22.0, "Undercity", 2 ) -- zone: Thunder Bluff
	AddTrainer( 28702, "Professor Pallin", 4, 3, 41.7, 37.3, "Kirin Tor", 3 ) -- zone: Dalaran
--	AddTrainer( 18804, "Prospector Nachlan", 1, 5, 56.3, 54.3, "nil", 1 ) -- zone: Bloodmyst Isle
--	AddTrainer(  3067, "Pyall Silentstride", 1, 15, 45.5, 58.1, "nil", 2 ) -- zone: Mulgore
--	AddTrainer( 16277, "Quarelestra", 2, 11, 48.6, 47.1, "nil", 2 ) -- zone: Eversong Woods
	AddTrainer( 16652, "Quithas", 2, 24, 58.6, 18.8, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer( 26969, "Raenah", 4, 1, 41.6, 53.5, "nil", 2 ) -- zone: Borean Tundra
	AddTrainer( 50157, "Ranger Selone", 4, 3, 38.6, 63.7, "The Silver Covenant", 1 ) -- zone: Dalaran
	AddTrainer(  5943, "Rawrk", 1, 9, 54.1, 42.0, "Orgrimmar", 2 ) -- zone: Durotar
	AddTrainer( 33679, "Recorder Lidio", 3, 6, 36.3, 43.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer( 16729, "Refik", 1, 23, 63.9, 69.0, "Exodar", 1 ) -- zone: The Exodar
	AddTrainer( 33591, "Rekka the Hammer", 4, 8, 71.9, 20.9, "Argent Crusade", 3 ) -- zone: Icecrown
	AddTrainer( 44782, "Rento", 1, 17, 39.8, 49.8, "Thunder Bluff", 2 ) -- zone: Orgrimmar
	AddTrainer( 48513, "Revi Ramrod", 1, 17, 36.5, 87.1, "Bilgewater Cartel", 2 ) -- zone: Orgrimmar
	AddTrainer(  5137, "Reyna Stonebranch", 2, 14, 55.8, 59.1, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 34712, "Roberta Carter", 2, 35, 64.0, 9.6, "nil", 2 ) -- zone: Undercity
	AddTrainer( 37072, "Rogg", 1, 17, 44.3, 77.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
--	AddTrainer(  1386, "Rogvar", 2, 28, 48.4, 55.7, "nil", 2 ) -- zone: Swamp of Sorrows
--	AddTrainer( 16583, "Rohok", 3, 2, 53.2, 38.2, "nil", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 26989, "Rollick MacKreel", 4, 1, 57.9, 71.5, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer( 26998, "Rosemary Bovard", 4, 1, 57.6, 71.9, "Valiance Expedition", 1 ) -- zone: Borean Tundra
--	AddTrainer( 26904, "Rosina Rivet", 4, 6, 59.6, 63.7, "nil", 1 ) -- zone: Howling Fjord
--	AddTrainer( 31247, "Roxi Ramrocket", 4, 10, 40.8, 84.8, "nil", 1 ) -- zone: The Storm Peaks
	AddTrainer( 11017, "Roxxik", 1, 17, 57.0, 56.4, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 50160, "Rulen Lightsreap", 4, 3, 39.0, 62.9, "The Silver Covenant", 1 ) -- zone: Dalaran
	AddTrainer( 13417, "Sagorne Creststrider", 1, 17, 39.7, 47.2, "Thunder Bluff", 2 ) -- zone: Orgrimmar
	AddTrainer( 44740, "Sahi Cloudsinger", 1, 17, 44.7, 52.3, "Thunder Bluff", 2 ) -- zone: Orgrimmar
--	AddTrainer( 26956, "Sally Tompkins", 4, 6, 79.4, 29.4, "nil", 2 ) -- zone: Howling Fjord
	AddTrainer(  5496, "Sandahl", 2, 26, 39.5, 85.5, "Stormwind", 1 ) -- zone: Stormwind City
--	AddTrainer(  7868, "Sarah Tanner", 2, 22, 63.7, 75.7, "nil", 1 ) -- zone: Searing Gorge
	AddTrainer(  3355, "Saru Steelfury", 1, 17, 76.4, 34.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
--	AddTrainer( 16278, "Sathein", 2, 11, 53.5, 51.0, "nil", 2 ) -- zone: Eversong Woods
	AddTrainer( 33638, "Scribe Lanloer", 3, 6, 56.2, 74.4, "The Scryers", 2 ) -- zone: Shattrath City
--	AddTrainer(  7871, "Se'Jib", 2, 27, 36.6, 34.2, "nil", 2 ) -- zone: Stranglethorn Vale
	AddTrainer( 16633, "Sedana", 2, 24, 70.0, 24.0, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer( 16366, "Sempstress Ambershine", 2, 11, 37.4, 71.9, "nil", 2 ) -- zone: Eversong Woods
--	AddTrainer(  2391, "Serge Hinott", 2, 13, 61.6, 19.2, "nil", 2 ) -- zone: Hillsbrad Foothills
	AddTrainer(  2327, "Shaina Fuller", 2, 26, 52.1, 45.6, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 44726, "Shalla Whiteleaf", 1, 17, 44.9, 51.6, "Thunder Bluff", 2 ) -- zone: Orgrimmar
--	AddTrainer(  7230, "Shayis Steelfury", 1, 17, 80.3, 23.2, "nil", 2 ) -- zone: Orgrimmar
	AddTrainer(  3034, "Sheal Runetotem", 1, 25, 76.6, 27.4, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  5504, "Sheldras Moontree", 2, 26, 27.5, 50.1, "Stormwind", 1 ) -- zone: Stormwind City
--	AddTrainer(  3549, "Shelene Rhobart", 2, 31, 65.5, 61.0, "nil", 2 ) -- zone: Tirisfal Glades
	AddTrainer(  3403, "Sian'tsu", 1, 17, 39.2, 47.8, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  4089, "Sildanair", 1, 7, 58.8, 50.7, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer(  3030, "Siln Skychaser", 1, 25, 23.1, 20.7, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  5564, "Simon Tanner", 2, 26, 71.8, 62.8, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 38516, "Sister Goldskimmer", 5, 3, 42.6, 16.5, "Bilgewater Cartel", 2 ) -- zone: The Lost Isles
	AddTrainer( 33618, "Skinning", 3, 6, 43.5, 91.1, "The Scryers", 2 ) -- zone: Shattrath City
--	AddTrainer(  2818, "Slagg", 2, 2, 74.1, 33.8, "nil", 2 ) -- zone: Arathi Highlands
--	AddTrainer( 21494, "Smiles O'Byron", 3, 1, 60.3, 65.2, "nil", 3 ) -- zone: Blade's Edge Mountains
	AddTrainer(   514, "Smith Argus", 2, 10, 41.7, 65.6, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 26991, "Sock Brightbolt", 4, 1, 57.7, 72.1, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer(  3354, "Sorek", 1, 17, 73.6, 45.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  5174, "Springspindle Fizzlegear", 2, 14, 68.7, 44.3, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  5482, "Stephen Ryback", 2, 26, 77.3, 53.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 50156, "Summoner Nolric", 4, 3, 38.4, 64.2, "The Silver Covenant", 1 ) -- zone: Dalaran
	AddTrainer( 44725, "Sunwalker Atohmo", 1, 17, 45.2, 53.6, "Thunder Bluff", 2 ) -- zone: Orgrimmar
	AddTrainer(  3173, "Swart", 1, 9, 54.4, 42.5, "Orgrimmar", 2 ) -- zone: Durotar
	AddTrainer( 16676, "Sylann", 2, 24, 69.5, 71.5, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  4163, "Syurna", 1, 7, 41.4, 37.0, "Darnassus", 1 ) -- zone: Darnassus
	AddTrainer( 33613, "Tailoring", 3, 6, 43.8, 90.9, "The Scryers", 3 ) -- zone: Shattrath City
--	AddTrainer(  4213, "Taladan", 1, 7, 58.6, 13.2, "nil", 1 ) -- zone: Darnassus
	AddTrainer(  5177, "Tally Berryfizz", 2, 14, 66.8, 55.0, "Gnomeregan", 1 ) -- zone: Ironforge
	AddTrainer(  5566, "Tannysa", 2, 26, 54.3, 84.2, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer( 50690, "Tarelvir", 1, 7, 37.7, 80.0, "Darnassus", 1 ) -- zone: Darnassus
--	AddTrainer( 18774, "Tatiana", 3, 2, 54.6, 63.6, "nil", 1 ) -- zone: Hellfire Peninsula
	AddTrainer( 33677, "Technician Mihila", 3, 6, 37.6, 31.9, "The Aldor", 3 ) -- zone: Shattrath City
	AddTrainer(  3011, "Teg Dawnstrider", 1, 25, 45.1, 38.4, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
--	AddTrainer(  4212, "Telonis", 1, 7, 64.5, 21.3, "nil", 1 ) -- zone: Darnassus
	AddTrainer(  1458, "Telurinon Moonshadow", 2, 39, 11.3, 52.1, "Ironforge", 1 ) -- zone: Wetlands
	AddTrainer(  3004, "Tepa", 1, 25, 44.3, 45.0, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  2326, "Thamner Pol", 2, 7, 54.3, 50.9, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer( 32712, "The Amazing Zanzo", 4, 3, 47.2, 36.9, "Kirin Tor", 1 ) -- zone: Dalaran
	AddTrainer( 44582, "Theresa Denman", 2, 26, 63.4, 61.7, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  5511, "Therum Deepforge", 2, 26, 63.5, 36.8, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  5171, "Thistleheart", 2, 14, 50.9, 6.8, "Ironforge", 1 ) -- zone: Ironforge
--	AddTrainer( 26953, "Thomas Kolichio", 4, 6, 78.6, 29.4, "nil", 2 ) -- zone: Howling Fjord
--	AddTrainer(  7867, "Thorkaf Dragoneye", 2, 3, 62.6, 57.6, "nil", 2 ) -- zone: Badlands
--	AddTrainer( 30716, "Thoth", 1, 23, 40.5, 39.1, "nil", 1 ) -- zone: The Exodar
	AddTrainer(   912, "Thran Khorman", 2, 7, 35.9, 65.7, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  3049, "Thurston Xane", 1, 25, 25.8, 21.8, "Undercity", 2 ) -- zone: Thunder Bluff
	AddTrainer(  5958, "Thuul", 1, 17, 38.6, 85.7, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  7088, "Thuwd", 1, 17, 60.9, 54.6, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer(  3031, "Tigor Skychaser", 1, 25, 23.0, 18.8, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 28697, "Timofey Oshenko", 4, 3, 38.9, 26.1, "Kirin Tor", 3 ) -- zone: Dalaran
	AddTrainer( 28701, "Timothy Jones", 4, 3, 40.3, 35.2, "Kirin Tor", 3 ) -- zone: Dalaran
--	AddTrainer( 11052, "Timothy Worthington", 1, 10, 66.2, 51.7, "nil", 1 ) -- zone: Dustwallow Marsh
	AddTrainer( 26995, "Tink Brightbolt", 4, 1, 57.6, 71.7, "Valiance Expedition", 1 ) -- zone: Borean Tundra
--	AddTrainer(  7944, "Tinkmaster Overspark", 2, 14, 69.8, 50.0, "nil", 1 ) -- zone: Ironforge
--	AddTrainer( 26907, "Tisha Longbridge", 4, 6, 59.7, 64.0, "nil", 1 ) -- zone: Howling Fjord
	AddTrainer(  1241, "Tognus Flintfire", 2, 7, 52.4, 50.3, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  5143, "Toldren Deepiron", 2, 14, 25.2, 10.2, "Ironforge", 1 ) -- zone: Ironforge
	AddTrainer(  1430, "Tomas", 2, 10, 44.3, 66.0, "Stormwind", 1 ) -- zone: Elwynn Forest
	AddTrainer( 27000, "Trapper Jack", 4, 1, 57.6, 71.9, "Valiance Expedition", 1 ) -- zone: Borean Tundra
	AddTrainer(  3033, "Turak Runetotem", 1, 25, 76.2, 27.8, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer( 45337, "Tyelis", 1, 17, 49.2, 71.0, "Silvermoon City", 2 ) -- zone: Orgrimmar
	AddTrainer(  4772, "Ultham Ironhorn", 2, 7, 71.1, 48.5, "Ironforge", 1 ) -- zone: Dun Morogh
	AddTrainer(  3007, "Una", 1, 25, 41.8, 42.7, "Thunder Bluff", 2 ) -- zone: Thunder Bluff
	AddTrainer(  5495, "Ursula Deline", 2, 26, 40.0, 84.3, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  3048, "Ursyn Ghull", 1, 25, 27.6, 17.6, "Undercity", 2 ) -- zone: Thunder Bluff
--	AddTrainer(  5695, "Vance Undergloom", 2, 31, 61.7, 51.6, "nil", 2 ) -- zone: Tirisfal Glades
--	AddTrainer(  5939, "Vira Younghoof", 1, 15, 46.8, 60.8, "nil", 2 ) -- zone: Mulgore
--	AddTrainer(  1246, "Vosur Brakthel", 2, 14, 66.5, 55.2, "nil", 1 ) -- zone: Ironforge
	AddTrainer( 28956, "Warcaster Fanoraithe", 4, 3, 55.2, 47.8, "Kirin Tor", 1 ) -- zone: Dalaran
	AddTrainer(  1683, "Warg Deepwater", 2, 17, 40.6, 39.5, "Ironforge", 1 ) -- zone: Loch Modan
	AddTrainer( 33684, "Weaver Aoa", 3, 6, 37.5, 27.3, "The Aldor", 3 ) -- zone: Shattrath City
--	AddTrainer( 26951, "Wilhelmina Renel", 4, 6, 78.7, 28.5, "nil", 2 ) -- zone: Howling Fjord
	AddTrainer( 35093, "Wind Rider Jahubo", 3, 2, 54.2, 41.6, "Orgrimmar", 2 ) -- zone: Hellfire Peninsula
	AddTrainer( 50127, "Windstalker Ifram", 3, 6, 59.3, 46.3, "The Consortium", 1 ) -- zone: Shattrath City
	AddTrainer( 45029, "Witch Doctor Umbu", 1, 17, 34.2, 64.5, "Darkspear Trolls", 2 ) -- zone: Orgrimmar
	AddTrainer(  5479, "Wu Shen", 2, 26, 80.1, 69.9, "Stormwind", 1 ) -- zone: Stormwind City
	AddTrainer(  7953, "Xar'Ti", 1, 9, 55.3, 75.5, "Darkspear Trolls", 2 ) -- zone: Durotar
--	AddTrainer(  7949, "Xylinnia Starshine", 1, 12, 31.6, 44.3, "nil", 1 ) -- zone: Feralas
--	AddTrainer( 19576, "Xyrol", 3, 4, 32.5, 66.7, "nil", 3 ) -- zone: Netherstorm
--	AddTrainer(  5392, "Yarr Hammerstone", 2, 7, 50.0, 50.3, "nil", 1 ) -- zone: Dun Morogh
	AddTrainer(  3347, "Yelmak", 1, 17, 55.7, 46.0, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 27703, "Ysuria", 1, 10, 66.0, 49.0, nil, 1 ) -- zone: Dustwallow Marsh
	AddTrainer( 16651, "Zaedana", 2, 24, 57.6, 19.6, "Silvermoon City", 2 ) -- zone: Silvermoon City
	AddTrainer(  3399, "Zamja", 1, 17, 32.3, 69.3, "Darkspear Trolls", 2 ) -- zone: Orgrimmar
	AddTrainer( 30710, "Zantasia", 2, 24, 69.5, 24.0, "Silvermoon City", 2 ) -- zone: Silvermoon City
--	AddTrainer( 14742, "Zap Farflinger", 1, 28, 61.2, 37.6, "nil", 3 ) -- zone: Winterspring
	AddTrainer( 45550, "Zarbo Porkpatty", 1, 17, 39.1, 86.2, "Bilgewater Cartel", 2 ) -- zone: Orgrimmar
--	AddTrainer(  6286, "Zarrin", 1, 22, 57.1, 61.3, "nil", 1 ) -- zone: Teldrassil
--	AddTrainer( 18752, "Zebig", 3, 2, 54.8, 38.5, "nil", 2 ) -- zone: Hellfire Peninsula
	AddTrainer(  3408, "Zel'mak", 1, 17, 80.2, 29.4, "Orgrimmar", 2 ) -- zone: Orgrimmar
	AddTrainer( 20125, "Zula Slagfury", 3, 6, 70.0, 42.4, "Lower City", 3 ) -- zone: Shattrath City
	AddTrainer( 33676, "Zurii", 3, 6, 36.5, 44.7, "The Aldor", 3 ) -- zone: Shattrath City

	self:FixTrainerRef()

end

-------------------------------------------------------------------------------
-- Trainner reference (in learnable spells)
-------------------------------------------------------------------------------

function Syllabus:AddTrainerRef(type_name, skill_name, spell_id, ...)
	local skills_db = Syllabus.db.global.skills
	local trainers_db = Syllabus.db.global.trainers

	local local_skill_name = type_name == "professions" and GetLocalProfName(skill_name) or skill_name
	local spell
	if type_name == "all_classes"then
		spell = skills_db[type_name][spell_id]
	else
		spell = skills_db[type_name][local_skill_name][spell_id]
	end

	if IsTableEmpty(spell) then
		err("Spell doesn't exists in the database: %s - %s - %s", type_name, skill_name, spell_id)
		return
	end

	for i=1,select('#',...) do
		local trainer_id = select(i,...)

		if type(trainer_id) ~= "number" then
			err("Invalid trainer_id %s for spell %s - %s - %s", trainer_id, type_name, skill_name, spell_id)
		elseif IsTableEmpty(trainers_db[trainer_id]) then
			err("Trainer %s doesn't exists for spell %s - %s - %s", trainer_id, type_name, skill_name, spell_id)
		else
			spell.trainers[trainer_id] = true

			-- Get the reputation and the faction from the trainer
			spell.valid_rep[trainers_db[trainer_id].rep] = true
			spell.is_alliance = trainers_db[trainer_id].is_alliance
			spell.is_horde = trainers_db[trainer_id].is_horde
		end
	end
end
